# '{{ project_name }}' on NimbleBox

* created at: {{ created_time }}

This is an autogenerated README file for a [NBX](www.nimblebox.ai) app.

Steps to complete:

1. Add your code/folders/csv/etc. in this folder and fill requirements.txt with all the dependencies
2. Fill `nbx_user.py:get_op` and then run `{{ upload_command }}`
3. Sit back, relax, and strap on your seat belt. - Dr. Dre

