"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import proto.relics_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CreateRelicRequest(google.protobuf.message.Message):
    """things related to general WebAPIs"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    WORKSPACE_ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    workspace_id: typing.Text
    """the main separation layer"""

    name: typing.Text
    region: typing.Text
    def __init__(self,
        *,
        workspace_id: typing.Text = ...,
        name: typing.Text = ...,
        region: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name",b"name","region",b"region","workspace_id",b"workspace_id"]) -> None: ...
global___CreateRelicRequest = CreateRelicRequest

class ListRelicsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    WORKSPACE_ID_FIELD_NUMBER: builtins.int
    RELIC_NAME_FIELD_NUMBER: builtins.int
    PAGE_NO_FIELD_NUMBER: builtins.int
    workspace_id: typing.Text
    """the main separation layer"""

    relic_name: typing.Text
    """the name of the relic"""

    page_no: builtins.int
    """the page number for pagination needs"""

    def __init__(self,
        *,
        workspace_id: typing.Text = ...,
        relic_name: typing.Text = ...,
        page_no: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["page_no",b"page_no","relic_name",b"relic_name","workspace_id",b"workspace_id"]) -> None: ...
global___ListRelicsRequest = ListRelicsRequest

class ListRelicsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    RELICS_FIELD_NUMBER: builtins.int
    TOTAL_RELICS_FIELD_NUMBER: builtins.int
    @property
    def relics(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[proto.relics_pb2.Relic]: ...
    total_relics: builtins.int
    """the total number of relics used for pagination"""

    def __init__(self,
        *,
        relics: typing.Optional[typing.Iterable[proto.relics_pb2.Relic]] = ...,
        total_relics: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["relics",b"relics","total_relics",b"total_relics"]) -> None: ...
global___ListRelicsResponse = ListRelicsResponse

class ListRelicFilesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    WORKSPACE_ID_FIELD_NUMBER: builtins.int
    RELIC_ID_FIELD_NUMBER: builtins.int
    RELIC_NAME_FIELD_NUMBER: builtins.int
    PREFIX_FIELD_NUMBER: builtins.int
    FILE_NAME_FIELD_NUMBER: builtins.int
    PAGE_NO_FIELD_NUMBER: builtins.int
    workspace_id: typing.Text
    """the main separation layer"""

    relic_id: typing.Text
    """the id of the relic"""

    relic_name: typing.Text
    """the id of the relic"""

    prefix: typing.Text
    """the prefix for the files"""

    file_name: typing.Text
    """for searching through the files, we may not support this functionality
    right now, but this is a crucial system to have
    """

    page_no: builtins.int
    """the page number for pagination needs"""

    def __init__(self,
        *,
        workspace_id: typing.Text = ...,
        relic_id: typing.Text = ...,
        relic_name: typing.Text = ...,
        prefix: typing.Text = ...,
        file_name: typing.Text = ...,
        page_no: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["file_name",b"file_name","page_no",b"page_no","prefix",b"prefix","relic_id",b"relic_id","relic_name",b"relic_name","workspace_id",b"workspace_id"]) -> None: ...
global___ListRelicFilesRequest = ListRelicFilesRequest

class ListRelicFilesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FILES_FIELD_NUMBER: builtins.int
    TOTAL_FILES_FIELD_NUMBER: builtins.int
    @property
    def files(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[proto.relics_pb2.RelicFile]: ...
    total_files: builtins.int
    """the total number of files used for pagination"""

    def __init__(self,
        *,
        files: typing.Optional[typing.Iterable[proto.relics_pb2.RelicFile]] = ...,
        total_files: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["files",b"files","total_files",b"total_files"]) -> None: ...
global___ListRelicFilesResponse = ListRelicFilesResponse
